#!/usr/bin/perl

package Class::Workflow::Environment;

use strict;
use warnings;



__PACKAGE__;

__END__

=pod

=head1 NAME

Class::Workflow::Environment - Tying instances together for synchronization.

=head1 SYNOPSIS

	use Class::Workflow::Environment;

=head1 DESCRIPTION

=cut


